
def split_imports_py(src_code):
    lines = src_code.splitlines()
    import_lines = []
    break_index = -1
    for i in range(0, len(lines)):
        if len(lines[i]) == 0 or lines[i][0] == '#':
            continue
        if lines[i][0:7] != 'import ' and lines[i][0:5] != 'from ':
            break_index = i
            break
        import_lines.append(lines[i])
    sourced_imports = {}
    aliased_imports = {}
    for import_line in import_lines:
        if import_line[0:7] == 'import ':
            L1 = [
                            [ z.strip() for z in y.strip().split(' as ') ]
                            for y in x.lstrip()[7:].strip().split(',')
                        ]
            for L2 in L1:
                if len(L2) == 1:
                    aliased_imports[L2[0]] = L2[0]
                    continue
                assert len(L2) == 2
                aliased_imports[L2[0]] = L2[1]
                continue
            continue
        if import_line[0:5] == 'from ':
            L0 = [
                            x.strip()
                            for x in import_line[5:].strip().split(' import ')
                        ]
            assert len(L0) == 2
            module_name = L0[0]
            L1 = [
                            [ z.strip() for z in y.strip().split(' as ') ]
                            for y in L0[1].split(',')
                        ]
            sourced_imports[module_name] = {}
            for L2 in L1:
                if len(L2) == 1:
                    sourced_imports[module_name][L2[0]] = L2[0]
                    continue
                sourced_imports[module_name][L2[0]] = L2[1]
                continue
            continue
    code_minus_imports_L1 = lines[break_index:]
    code_minus_imports = '\n'.join(code_minus_imports_L1)
    return sourced_imports, aliased_imports, code_minus_imports


def split_imports_typ(src_code):
    lines = src_code.splitlines()
    import_lines = []
    break_index = -1
    for i in range(0, len(lines)):
        if len(lines[i]) == 0 or lines[i][0] == '#':
            continue
        if lines[i][0:6] != 'import' and lines[i][0:7] != 'include':
            break_index = i
            break
        import_lines.append(lines[i])
    include_set = {
            x.lstrip()[8:].strip() : 1
            for x in import_lines
            if x.lstrip()[0:8] == 'include '
        }
    aliased_imports = {
            module_name : module_alias
            for x in import_lines
            if x.lstrip()[0:7] == 'import '
            for module_name, module_alias in [
                [
                    y.strip()
                    for y in x.lstrip()[7:].strip().split(' as ')
                ]
            ]
        }
    code_minus_imports_L1 = lines[break_index:]
    code_minus_imports = '\n'.join(code_minus_imports_L1)
    return include_set, aliased_imports, code_minus_imports


def join_imports_py(sourced_imports, aliased_imports, code_minus_imports):
    import_repr_L1 = [
            f'from {module_name} import {module_item} as {module_alias}'
            for module_name in sourced_imports
            for module_item in sourced_imports[module_name]
            for module_alias in [ sourced_imports[module_name][module_item] ]
        ] + [
            f'import {module_name} as {module_alias}'
            for module_name in aliased_imports
            for module_alias in [ aliased_imports[module_name] ]
        ]
    import_repr = '\n'.join(import_repr_L1)
    return f'''
    {import_repr}
    {code_minus_imports}
    '''


def join_imports_typ(include_set, aliased_imports, code_minus_imports):
    import_repr_L1 = [
            f'include {c_ext_name}'
            for c_ext_name in include_set
        ] + [
            f'import {module_name} as {module_alias}'
            for module_name in aliased_imports
            for module_alias in [ aliased_imports[module_name] ]
        ]
    import_repr = '\n'.join(import_repr_L1)
    return f'''
    {import_repr}
    {code_minus_imports}
    '''



import json
import lib_py_parse.utils.exceptions as exceptions
import lib_py_parse.utils.parsing_utils as parsing_utils

def parse_def(code, index, scope):
    K1 = 4
    assert code[index:index+K1] == 'def '
    i1 = parsing_utils.read_next_cleared_char(code, index, ':')
    if i1 == -1:
        exceptions.raise_exception_msg(code, index, 'Invalid fxn declaration, closing colon not found')
    s1 = code[index:i1]
    i2 = s1.find('(', 0)
    if i2 == -1:
        exceptions.raise_exception_msgs(code, index, ['Invalid fxn declaration, parenthesis not found', f'Line: {s1}'])
    name = s1[K1:i2].strip()
    arguments = s1[i2:].strip()
    statements, end_index = read_scope(code, i1+1, scope+1)
    result = {
            'type' : 'fxn',
            'index' : index,
            'name' : name,
            'arguments' : arguments,
            'statements' : statements,
        }
    return result, end_index


def parse_class(code, index, scope):
    K1 = 6
    assert code[index:index+K1] == 'class '
    i1 = parsing_utils.read_next_cleared_char(code, index, ':')
    if i1 == -1:
        exceptions.raise_exception_msg(code, index, 'Invalid class declaration, closing colon not found')
    s1 = code[index:i1]
    i2 = s1.find('(', 0)
    if i2 == -1:
        exceptions.raise_exception_msg(code, index, 'Invalid class declaration, parenthesis not found')
    name = s1[K1:i2].strip()
    inherits = s1[i2:].strip()
    statements, end_index = read_scope(code, i1+1, scope+1)
    result = {
            'type' : 'class',
            'index' : index,
            'name' : name,
            'inherits' : inherits,
            'statements' : statements,
        }
    return result, end_index


def parse_try(code, index, scope):
    K1 = 4
    assert code[index:index+K1] == 'try:'
    i1 = parsing_utils.read_next_cleared_char(code, index, ':')
    if i1 == -1:
        exceptions.raise_exception_msg(code, index, 'Invalid try declaration, closing colon not found')
    statements, end_index = read_scope(code, i1+1, scope+1)
    result = {
            'type' : 'try',
            'index' : index,
            'statements' : statements,
        }
    return result, end_index


def parse_except(code, index, scope):
    K1 = 7
    assert code[index:index+K1] == 'except:' or code[index:index+K1] == 'except '
    i1 = parsing_utils.read_next_cleared_char(code, index, ':')
    if i1 == -1:
        exceptions.raise_exception_msg(code, index, 'Invalid class declaration, closing colon not found')
    s1 = code[index:i1]
    statements, end_index = read_scope(code, i1+1, scope+1)
    result = {
            'type' : 'except',
            'index' : index,
            'declaration' : s1,
            'statements' : statements,
        }
    return result, end_index


def parse_L1_CE(code, index, scope, token):
    assert code[index:index+len(token)] == token
    i1 = parsing_utils.read_next_cleared_char(code, index+len(token), ':')
    if i1 == -1:
        exceptions.raise_exception_msg(code, index, f'Invalid {token} declaration, closing colon not found')
    declaration = code[index+len(token):i1].strip()
    statements, end_index = read_scope(code, i1+1, scope+1)
    result = {
            'type' : token,
            'index' : index,
            'declaration' : declaration,
            'statements' : statements,
        }
    return result, end_index


def parse_L0_statement(code, index, scope):
    i1 = parsing_utils.read_next_cleared_char(code, index, '\n')
    if i1 == -1:
        i1 = len(code)
    expression = code[index:i1]
    result = {
            'type' : 'instruction',
            'index' : index,
            'expression' : expression,
        }
    return result, i1


def parse_import(code, index, scope):
    assert code[index:index+6] == 'import'
    i1 = parsing_utils.read_next_cleared_char(code, index, '\n')
    if i1 == -1:
        i1 = len(code)
    result = {
            'type' : 'import',
            'index' : index,
            'expression' : code[index:i1],
        }
    return result, i1


def parse_include(code, index, scope):
    assert code[index:index+7] == 'include'
    i1 = parsing_utils.read_next_cleared_char(code, index, '\n')
    if i1 == -1:
        i1 = len(code)
    result = {
            'type' : 'include',
            'index' : index,
            'expression' : code[index:i1],
        }
    return result, i1


def parse_abstract_statement(code, index, scope, start_token):
    L2_router = {
            'class' : parse_class,
            'def' : parse_def,
        }
    L1_CE_router = {
            'for' : 1,
            'while' : 1,
            'with' : 1,
            'if' : 1,
            'elif' : 1,
            'else' : 1,
        }
    L1_special_items = {
            'try' : parse_try,
            'except' : parse_except,
        }
    L0_misc_router = {
            'import' : parse_import,
            'include' : parse_include,
        }
    if start_token in L2_router:
        return L2_router[start_token](code, index, scope)
    if start_token in L1_special_items:
        return L1_special_items[start_token](code, index, scope)
    if start_token in L1_CE_router:
        return parse_L1_CE(code, index, scope, start_token)
    if start_token in L0_misc_router:
        return L0_misc_router[start_token](code, index, scope)
    return parse_L0_statement(code, index, scope)


def read_scope(code, index, scope):
    result = []
    N = float('inf')
    ctr = 0
    while True and ctr < N:
        start_token, i1 = parsing_utils.read_init_token(code, index, scope)
        if start_token == '#':
            i1 = code.find('\n', i1)
            if i1 == -1:
                return result, len(code)
            index = i1
            continue
        if start_token in ['EOF', 'EOS']:
            return result, i1
        assert start_token == code[i1-len(start_token):i1]
        abstract_statement, index = parse_abstract_statement(code, i1 - len(start_token), scope, start_token)
        result.append(abstract_statement)
        ctr += 1
    raise Exception('parse timeout')




def select_item_prefix(item_name):
    item_router = {
            'fxn' : 'F',
            'fxn_param' : 'P',
            'namespace' : 'NS',
            'class' : 'C',
            'class_member' : 'A',
            'class_fxn' : 'F',
            'local_var' : 'T',
            'for_loop_index' : 'i',
            'key_value_index' : 'kv',
            'c_ext_type_prefix' : '@c.'
        }
    return item_router[item_name]


def select_primitives():
    ptype_map_V1 = {
            'void' : 'void',
            'fileptr' : 'FILE *',
            'thread' : 'std::thread',
            'semaphore' : 'semaphore *',
            'int' : 'long int',
            'uint32' : 'unsigned int',
            'uint64' : 'unsigned long int',
            'int32' : 'int',
            'dbl' : 'double',
            'char' : 'char',
            'bool' : 'bool',
            'str' : 'std::string',
        }
    return ptype_map_V1


def select_bracket_map():
    t_tokens = '[](){}'
    t_mapper = {
            t_tokens[2 * i] : t_tokens[2 * i + 1]
            for i in range(0, len(t_tokens) // 2)
        }
    return t_mapper




def select_prefix_closure( prefix ):
    buf = bytearray()
    for i in range( len(prefix)-1, -1, -1 ):
        if prefix[i] == 255:
            buf.append(0)
            continue
        X = bytearray()
        X.append( prefix[i] + 1 )
        return prefix[0:i] + bytes(X) + bytes(buf)
    return None


class ldb_range_iter():
    def __init__( self, db_ref, start, stop, include_start, include_stop, include_key, include_value, reverse ):
        self.db_it = db_ref.RangeIter(
                    key_from = start,
                    key_to = stop,
                    include_value = include_value,
                    reverse = reverse
                )
        self.start = start
        self.stop = stop
        self.include_start = include_start
        self.include_stop = include_stop
        self.include_key = include_key
        self.include_value = include_value
        self.reverse = reverse
        self.cur_kv = []
        self.val_set = False
        self.iter_finish = False
    def has_next(self):
        if self.iter_finish:
            return False
        if self.val_set:
            return True
        if not self.include_value:
            while True:
                X = next(self.db_it, None)
                if X == None:
                    self.iter_finish = True
                    return False
                key_bytes = bytes(X[0])
                if key_bytes == self.start and not self.include_start:
                    continue
                if key_bytes == self.stop and not self.include_stop:
                    continue
                if not self.include_key:
                    self.cur_kv = []
                    self.val_set = True
                    return True
                self.cur_kv = [ key_bytes ]
                self.val_set = True
                return True
            return False
        while True:
            X = next(self.db_it, None)
            if X == None:
                self.iter_finish = True
                return False
            key_bytes = bytes(X[0])
            value_bytes = bytes(X[1])
            if key_bytes == self.start and not self.include_start:
                continue
            if key_bytes == self.stop and not self.include_stop:
                continue
            if not self.include_key:
                self.cur_kv = [ value_bytes ]
                self.val_set = True
                return True
            self.cur_kv = [ key_bytes, value_bytes ]
            self.val_set = True
            return True
        return False
    def get_next(self):
        if not self.has_next():
            return None
        self.val_set = False
        return self.cur_kv


class ldb_prefix_iterator():
    def __init__( self, db_ref,  prefix, include_key, include_value, reverse  ):
        prefix_closure = select_prefix_closure( prefix )
        self.db_it = db_ref.RangeIter(
                    key_from = prefix,
                    key_to = prefix_closure,
                    include_value = include_value,
                    reverse = reverse
                )
        self.start = prefix
        self.stop = prefix_closure
        self.include_key = include_key
        self.include_value = include_value
        self.reverse = reverse
        self.cur_kv = []
        self.val_set = False
        self.iter_finish = False
    def has_next(self):
        if self.iter_finish:
            return False
        if self.val_set:
            return True
        if not self.include_value:
            while True:
                X = next(self.db_it, None)
                if X == None:
                    self.iter_finish = True
                    return False
                key_bytes = bytes(X[0])
                if self.prefix != key_bytes[0:len(self.prefix)]:
                    continue
                if not self.include_key:
                    self.cur_kv = []
                    self.val_set = True
                    return True
                self.cur_kv = [ key_bytes ]
                self.val_set = True
                return True
            return False
        while True:
            X = next(self.db_it, None)
            if X == None:
                self.iter_finish = True
                return False
            key_bytes = bytes(X[0])
            value_bytes = bytes(X[1])
            if self.prefix != key_bytes[0:len(self.prefix)]:
                continue
            if not self.include_key:
                self.cur_kv = [ value_bytes ]
                self.val_set = True
                return True
            self.cur_kv = [ key_bytes, value_bytes ]
            self.val_set = True
            return True
        return False
    def get_next(self):
        if not self.has_next():
            return None
        self.val_set = False
        return self.cur_kv


class ldb_iterator():
    def __init__(
            self,
            db_ref,
            start=None,
            stop=None,
            prefix=None,
            include_start=True,
            include_stop=False,
            include_key=True,
            include_value=True,
            reverse=False
        ):
        if prefix != None:
            self.base_it = ldb_prefix_iterator( db_ref, prefix, include_key, include_value, reverse )
            return
        self.base_it = ldb_range_iter( db_ref, start, stop, include_start, include_stop, include_key, include_value, reverse )
    def has_next(self):
        return self.base_it.has_next()
    def get_next(self):
        return self.base_it.get_next()



import json

def find_all(s, t, index):
    i1 = index-1
    result = []
    while True:
        i1 = s.find(t, i1+1)
        if i1 == -1:
            break
        result.append(i1)
    return result


def raise_exception_msg(code, index, error_msg):
    L1 = find_all(code[:index], '\n', 0)
    line_num = len(L1)
    formal_message = f'''
    Compiler Error:
        Line: {line_num}
        Message: {error_msg}
    '''
    raise Exception(formal_message)


def raise_exception_msgs(code, index, error_msgs):
    L1 = find_all(code[:index], '\n', 0)
    line_num = len(L1)
    error_msg = '\n'.join([f'        {x}' for x in error_msgs])
    formal_message = f'''
    Compiler Error:
        Line: {line_num}
        Messages:
    {error_msg}
    '''
    raise Exception(formal_message)


def raise_exception_raw(error_msgs):
    error_msg = '\n'.join([f'        {x}' for x in error_msgs])
    formal_message = f'''
    Compiler Error:
        Type: Semantic Error
        Messages:
    {error_msg}
    '''
    raise Exception(formal_message)


def raise_exception_ue(error_msgs):
    error_msg = '\n'.join([f'        {x}' for x in error_msgs])
    formal_message = f'''
    Compiler Error:
        Messages:
    {error_msg}
    '''
    raise Exception(formal_message)


def raise_exception_ue_cm(error_msgs, current_module):
    error_msg = '\n'.join([f'        {x}' for x in error_msgs])
    formal_message = f'''
    Compiler Error:
        current module: {current_module}
        Messages:
    {error_msg}
    '''
    raise Exception(formal_message)


def raise_exception_ST(symbol_table, errors):
    error_msg = ''
    if isinstance(errors, list):
        error_msg = '\n'.join([f'        {x}' for x in errors])
    else :
        error_msg = str(errors)
    code = symbol_table['current_module']['module_src']
    index = -1
    if len(symbol_table['current_module']['current_scope']) > 0:
        index = symbol_table['current_module']['current_scope'][-1]['index']
    L1 = code[:index].splitlines()
    L2 = code.splitlines()
    line_num = len(L1)
    file_name = symbol_table['current_module']['module_src_path']
    formal_message = f'''
    Compiler Error:
        File: {file_name}
        Line Number: {line_num}
        Line:
    {L2[line_num-1]}
        Messages:
    {error_msg}
    '''
    raise Exception(formal_message)


def format_parse_tree(abstract_tree):
    return json.dumps(abstract_tree, indent=1)


def raise_inv_exception_decl(exception_decl, symbol_table):
    error_msgs = [
            f'invalid exception declaration: {exception_decl}',
            'required format: except Exception as {typon_identifier}',
        ]
    raise_exception_ST(symbol_table, error_msgs)


def raise_invalid_raise_exception(symbol_table):
    error_msgs = [
            f'invalid use of raise keyword:',
            'required format: raise [Exception]([str])',
        ]
    raise_exception_ST(symbol_table, error_msgs)


def raise_class_not_found_exception(symbol_table, class_name, target_module):
    error_msgs = [
            f'unable to resolve class_name: {class_name}',
            f'located in target module: {target_module}'
        ]
    raise_exception_ST(symbol_table, error_msgs)


def raise_inv_assertion_declaration(symbol_table, interior_assert_typon_type):
    error_msgs = [
            f'invalid assertion declaration:',
            f'interior expression resolves to type: {interior_assert_typon_type}',
            f'when type bool is required',
        ]
    raise_exception_ST(symbol_table, error_msgs)


def raise_exception_L0_skeleton(errors, current_module, L0_skeleton):
    error_msg = ''
    if isinstance(errors, list):
        error_msg = '\n'.join([f'        {x}' for x in errors])
    else :
        error_msg = str(errors)
    formal_message = f'''
        Target Module: {current_module}
    {error_msg}
    '''
    raise Exception(formal_message)


def raise_parsing_exception(error_msg_str, current_module, src_dir):
    formal_message = f'''
    Error parsing module:
        target module: {current_module}
        source path: {src_dir}
    {error_msg_str}
    '''
    raise Exception(formal_message)



import os
import json
import lib_compute.graph_alg as graph_alg
import lib_py_parse.utils.fs_mod as fs_mod
import lib_py_parse.utils.parsing_utils as parsing_utils
import lib_py_parse.parse.parse_skeleton as parse_skeleton
import lib_py_parse.parse.parse_L0_skeleton as parse_L0_skeleton

def fmt_json(X):
    S1 = json.dumps(X, indent=4)
    print(S1)
    return S1


def render_indented_stmts( stmt ):
    indent_str = ' ' * 4
    stmts_L1 = []
    for x in stmt['statements']:
        S1 = render_stmt( x )
        stmts_L1.append( S1 )
    stmts_repr = '\n'.join(stmts_L1)
    stmts_repr = parsing_utils.indent_multiline_str(' ' * 4, stmts_repr)
    stmts_repr_L1 = [
            x
            for x in stmts_repr.splitlines()
            if len( x.strip() ) > 0
        ]
    stmts_repr = '\n'.join(stmts_repr_L1)
    return stmts_repr


def render_fxn_repr( stmt ):
    name = stmt['name']
    arguments = stmt['arguments']
    stmts_repr = render_indented_stmts( stmt )
    return f'''
    def {name}{arguments}:
    {stmts_repr}
    '''


def render_class_repr( stmt ):
    name = stmt['name']
    inherits = stmt['inherits']
    stmts_repr = render_indented_stmts( stmt )
    return f'''
    class {name}{inherits}:
    {stmts_repr}
    '''


def render_instruction_repr( stmt ):
    return stmt['expression']


def render_import_repr( stmt ):
    return stmt['expression']


def render_L1_control( stmt ):
    ctl = stmt['type']
    decl = stmt['declaration']
    stmts_repr = render_indented_stmts( stmt )
    return f'''
    {ctl} {decl}:
    {stmts_repr}
    '''


def render_try_repr( stmt ):
    ctl = stmt['type']
    stmts_repr = render_indented_stmts( stmt )
    return f'''
    {ctl}:
    {stmts_repr}
    '''


def render_except_repr( stmt ):
    ctl = stmt['type']
    stmts_repr = render_indented_stmts( stmt )
    return f'''
    {ctl}:
    {stmts_repr}
    '''


def render_stmt( stmt ):
    router = {
            type_name : eval( f'render_{type_name}_repr' )
            for type_name in [
                'fxn',
                'class',
                'instruction',
                'import',
                'try',
                'except',
            ]
        }
    if stmt['type'] in router:
        return router[ stmt['type'] ]( stmt )
    return render_L1_control( stmt )


def rm_comment_U1(mod_skeleton_U1):
    result = [
            render_stmt( stmt )
            for stmt in mod_skeleton_U1
        ]
    return '\n'.join(result)


def rm_comment_module( target_module, module_dir ):
    skeleton = parse_L0_skeleton.select_code_skeletons(target_module, module_dir)
    return rm_comment_U1(skeleton[target_module])


def rm_comment_pkg( pkg_ns, pkg_base ):
    target_module_L1 = fs_mod.iter_py_mod( pkg_ns, pkg_base )
    M1 = {
            target_module : [
                rm_comment_module( target_module, pkg_base ),
                fspath,
            ]
            for target_module, fspath in target_module_L1
        }
    return M1


def rm_comment_pkg_src( pkg_ns, pkg_base ):
    M1 = rm_comment_pkg( pkg_ns, pkg_base )
    for target_module in M1:
        rm_comment_src, fspath = M1[target_module]
        fp = open(fspath, 'w')
        fp.write(rm_comment_src)
        fp.close()
    return M1


def p1():
    target_module = 'lib_dsa.graph_db'
    module_dir = '/home/algorithmspath/vol1/py_lib_src'
    pkg_ns = 'lib_compute'
    pkg_base = '/home/algorithmspath/vol1/py_lib_src'
    result = rm_comment_pkg( pkg_ns, pkg_base )
    S1 = json.dumps(result, indent=4)
    print(S1)


def main():
    pass


if __name__ == '__main__':
    main()



import os, json

def bfs_files(base_dir, target_exts):
    target_exts_map = { x : 1 for x in target_exts }
    q = [base_dir]
    result = []
    while len(q) > 0:
        fspath = q.pop()
        if os.path.isdir(fspath):
            for file in os.listdir(fspath):
                q.append( f'{fspath}/{file}' )
            continue
        i1 = fspath.rfind('.', 0)
        ext = fspath[i1+1 : len(fspath) ]
        if ext in target_exts_map or len(target_exts) == 0:
            result.append(fspath)
    return result


def transforms( base_dir, target_exts ):
    abs_fspaths = bfs_files(base_dir, target_exts)
    rel_fspaths = [ x[len(base_dir)+1:] for x in abs_fspaths ]
    file_names = [ x[ x.rfind('/', 0) + 1 : ] for x in abs_fspaths ]
    return [ abs_fspaths, rel_fspaths, file_names ]


def read_imports(src_fspath):
    fp = open(src_fspath, 'r')
    fsource = fp.read()
    fp.close()
    lines = fsource.splitlines()
    index = len(lines)
    for i in range(0, len(lines)):
        line = lines[i]
        if len(line) > 0 and line[0] == '#':
            continue
        if line[0:7] != 'import ':
            index = i
            break
    import_list = []
    imports = lines[0:index]
    for line in imports:
        if len(line) > 0 and line[0] == '#':
            continue
        S1 = line[7:]
        L1 = [ x.strip() for x in S1.split(',')  ]
        import_list += [
                    x.strip() for x in import_seg.split(' as ')
                    for import_seg in L1
                ]
    return lines, index, import_list


def transform_imports(src_fspath, dst_fspath, src_pkg_name, dst_pkg_name, module_correspondence):
    fp = open(src_fspath, 'r')
    fsource = fp.read()
    fp.close()
    lines = fsource.splitlines()
    index = len(lines)
    lines, index, import_list = read_imports(src_fspath)
    revised_imports = []
    for L2 in import_list:
        if len(L2) == 1:
            revised_imports.append( f'import {L2[0]}' )
            continue
        if L2[0] not in module_correspondence:
            e_msgs_L1 = [
                            f'uncontained module collection',
                            f'mapping for package: {L2[0]} is undefined',
                        ]
            e_msg = '\n'.join(e_msgs_L1)
            raise Exception( e_msg )
        revised_imports.append( f'import {module_correspondence[L2[0]]} as {L2[1]}' )
    dst_repr = '\n'.join( revised_imports + lines[index:] )
    fp = open(dst_fspath, 'w')
    fp.write(dst_repr)
    fp.close()


def check_transform(module_space_map):
    module_paths = [
            [ x[ x.rfind('/', 0) + 1 : ], x ]
            for module_name in module_space_map
            for x in module_space_map[module_name]
        ]
    unq_file_names = {}
    unq_rel_fspaths = {}
    for file_name, rel_path in module_paths:
        if file_name in unq_file_names:
            print( file_name )
        if rel_path in unq_rel_fspaths:
            print( rel_path )
        unq_file_names[file_name] = 1
        unq_rel_fspaths[rel_path] = 1
    b1 = len( unq_rel_fspaths ) == len( module_paths )
    b2 = len( unq_file_names ) == len( module_paths )
    print( b1 )
    print( b2 )


def compute_module_correspondence( src_pkg_name, dst_pkg_name, module_transform ):
    result = {}
    L1 = [ x for module_name in module_transform for x in module_transform[module_name] ]
    result_values = {}
    for dst_module_name in module_transform:
        for rel_path in module_transform[dst_module_name]:
            I1 = rel_path.rfind('/', 0) + 1
            X1 = rel_path[ I1 : ]
            if X1[-3:] == '.py':
                X1 = X1[:-3]
            src_module_name = rel_path[ : I1-1 ].replace('/', '.')
            src_mod = f'{src_pkg_name}.{src_module_name}.{X1}'
            dst_mod = f'{dst_pkg_name}.{dst_module_name}.{X1}'
            if src_mod in result:
                e_msgs_L1 = [
                                    f'given module_mapping is not injective',
                                    f'{src_mod} is not uniquely defined',
                                ]
                e_msg = '\n'.join(e_msgs_L1)
                raise Exception( e_msg )
            if dst_mod in result_values:
                e_msgs_L1 = [
                                    f'given module_mapping is not surjective',
                                    f' F[{result_values[dst_mod]}] = F[{src_mod}] = {dst_mod}',
                                ]
                e_msg = '\n'.join(e_msgs_L1)
                raise Exception( e_msg )
            result[ src_mod ] = dst_mod
            result_values[ dst_mod ] = src_mod
    return result


def build_base_pkgs( dst_base_dir, module_transform ):
    for dst_pkg in module_transform:
        dst_fmt = dst_pkg.replace('.', '/')
        pkg_dir = f'{dst_base_dir}/{dst_fmt}'
        pkg_init = f'{dst_base_dir}/{dst_fmt}/__init__.py'
        if not os.path.isdir(pkg_dir):
            os.mkdir(pkg_dir)
        if os.path.exists(pkg_init):
            continue
        file_repr = ''
        fp = open(pkg_init, 'w')
        fp.write(file_repr)
        fp.close()


def assert_module_closure(src_base, dst_base, module_correspondence):
    src_rel = src_mod.replace('.', '/')
    dst_rel = module_correspondence[src_mod].replace('.', '/')
    src_fspath = f'{src_base}/{src_rel}.py'
    dst_fspath = f'{dst_base}/{dst_rel}.py'
    lines, index, import_list = read_imports(src_fspath)
    revised_imports = []
    for L2 in import_list:
        if len(L2) > 1 and L2[0] not in module_correspondence:
            e_msgs_L1 = [
                            f'uncontained module collection',
                            f'mapping for package: {L2[0]} is undefined',
                        ]
            e_msg = '\n'.join(e_msgs_L1)
            raise Exception( e_msg )


def eval_map( src_base, dst_base, src_pkg_name, dst_pkg_name, module_mapping ):
    dst_base_dir = f'{dst_base}/{dst_pkg_name}'
    build_base_pkgs( dst_base_dir, module_mapping )
    module_correspondence = compute_module_correspondence( src_pkg_name, dst_pkg_name, module_mapping )
    assert_module_closure(src_base, dst_base, module_correspondence)
    for src_mod in module_correspondence:
        src_rel = src_mod.replace('.', '/')
        dst_rel = module_correspondence[src_mod].replace('.', '/')
        src_fspath = f'{src_base}/{src_rel}.py'
        dst_fspath = f'{dst_base}/{dst_rel}.py'
        transform_imports(src_fspath, dst_fspath, src_pkg_name, dst_pkg_name, module_correspondence)



import lib_compute.typon_math as typon_math
import lib_py_parse.utils.parsing_utils as parsing_utils

def deserialize_data(target_buffer, target_type, buffer_index, classes, str_encoding=None):
    if buffer_index + 1 > len(target_buffer):
        return [None, -1]
    target_type = target_type.replace(' ', '')
    if target_type == 'bool':
        result = ( target_buffer[buffer_index] == chr(1) )
        return [ result, buffer_index + 1 ]
    if target_type == 'char':
        result = target_buffer[buffer_index]
        return [ result, buffer_index + 1 ]
    if target_type == 'int':
        if buffer_index + 8 > len(target_buffer):
            return [None, -1]
        result = typon_math.deserialize_int(target_buffer[ buffer_index : buffer_index + 8])
        return [ result, buffer_index + 8 ]
    if target_type == 'dbl':
        if buffer_index + 8 > len(target_buffer):
            return [None, -1]
        result = typon_math.deserialize_dbl(target_buffer[ buffer_index : buffer_index + 8])
        return [ result, buffer_index + 8 ]
    if target_type == 'str':
        if buffer_index + 8 > len(target_buffer):
            return [None, -1]
        str_len = typon_math.deserialize_int(target_buffer[ buffer_index : buffer_index + 8])
        if buffer_index + 8 + str_len > len(target_buffer):
            return [None, -1]
        result = target_buffer[ buffer_index + 8 : buffer_index + 8 + str_len ]
        if str_encoding != None:
            result = result.decode(str_encoding)
        return [ result, buffer_index + 8 + str_len]
    if target_type[0:4] == 'vec[':
        if buffer_index + 8 > len(target_buffer):
            return [None, -1]
        item_type = target_type[4:-1]
        vec_len = typon_math.deserialize_int(target_buffer[ buffer_index : buffer_index + 8])
        result = []
        buffer_index_vec = buffer_index + 8
        for i in range(0, vec_len):
            value, buffer_index_vec = deserialize_data( target_buffer, item_type, buffer_index_vec, classes, str_encoding=str_encoding )
            if buffer_index_vec == -1:
                return [None, -1]
            result.append(value)
        return [ result, buffer_index_vec ]
    if target_type[0:5] in ['hmap[', 'smap[']:
        if buffer_index + 8 > len(target_buffer):
            return [None, -1]
        delimited_types = target_type[5:-1]
        key_type, value_type = parsing_utils.read_delimited_types(delimited_types)
        map_len = typon_math.deserialize_int(target_buffer[ buffer_index : buffer_index + 8])
        result = {}
        buffer_index_map = buffer_index + 8
        for i in range(0, map_len):
            key_ref, buffer_index_map = deserialize_data( target_buffer, key_type, buffer_index_map, classes, str_encoding=str_encoding )
            if buffer_index_map == -1:
                return [None, -1]
            value_ref, buffer_index_map = deserialize_data( target_buffer, value_type, buffer_index_map, classes, str_encoding=str_encoding )
            if buffer_index_map == -1:
                return [None, -1]
            result[key_ref] = value_ref
        return [ result, buffer_index_map ]
    if target_type[0:5] == 'tupl[':
        delimited_types = target_type[5:-1]
        typon_tuple_types = parsing_utils.read_delimited_types(delimited_types)
        result = []
        buffer_index_tupl = buffer_index
        for i in range(0, len(typon_tuple_types)):
            value_ref, buffer_index_tupl = deserialize_data( target_buffer, typon_tuple_types[i], buffer_index_tupl, classes, str_encoding=str_encoding )
            if buffer_index_tupl == -1:
                return [None, -1]
            result.append(value_ref)
        return [ result, buffer_index_tupl ]
    if target_type in classes:
        class_items = classes[target_type]
        buffer_index_class = buffer_index
        constructor_args = []
        for attr_name in class_items:
            attr_type = class_items[attr_name]
            attr_ref, buffer_index_class = deserialize_data( target_buffer, attr_type, buffer_index_class, classes, str_encoding=str_encoding )
            if buffer_index_class == -1:
                return [None, -1]
            constructor_args.append(attr_ref)
        result = eval( f'{target_type}(*constructor_args)')
        return [ result, buffer_index_class ]
    raise Exception(f'unverified type in fxn deserialize_data: {target_type}')



import json
import lib_py_parse.helper.type_resolver as type_resolver
import lib_py_parse.parse.parse_L0_skeleton as parse_L0_skeleton
import lib_py_parse.utils.fs_mod as fs_mod
import lib_py_parse.utils.parsing_utils as parsing_utils
import lib_compute.graph_alg as graph_alg

def gen_mod_repr( mod_skeleton_U1 ):
    U1_repr_L1 = []
    for stmt in mod_skeleton_U1:
        if stmt['type'] == 'fxn':
            X = 'def ' + stmt['name'] + stmt['arguments']
            U1_repr_L1.append( X )
            U1_repr_L1.append( '' )
            continue
        if stmt['type'] == 'class':
            X = 'class ' + stmt['name'] + stmt['inherits']
            U1_repr_L1.append( X )
            for stmt2 in stmt['statements']:
                if stmt2['type'] == 'fxn':
                    X2 = ' ' * 4 + 'def ' + stmt2['name'] + stmt2['arguments']
                    U1_repr_L1.append( X2 )
                    U1_repr_L1.append( '' )
                    continue
            U1_repr_L1.append( '' )
    return '\n'.join( U1_repr_L1 )


def generate_api_def(target_module, module_dir, target_only=False):
    mod_skeleton = parse_L0_skeleton.select_code_skeletons(target_module, module_dir)
    api_def_L1 = []
    if not target_only:
        for module_name in mod_skeleton:
            mod_repr = gen_mod_repr( mod_skeleton[module_name] )
            mod_repr = parsing_utils.indent_multiline_str(' ' * 4, mod_repr)
            api_def_L1.append( f'{module_name}' )
            api_def_L1.append( '' )
            api_def_L1.append( mod_repr )
            api_def_L1.append( '' )
        api_def = '\n'.join(api_def_L1)
        return api_def
    mod_repr = gen_mod_repr( mod_skeleton[target_module] )
    mod_repr = parsing_utils.indent_multiline_str(' ' * 4, mod_repr)
    api_def_L1.append( f'{target_module}' )
    api_def_L1.append( '' )
    api_def_L1.append( mod_repr )
    api_def_L1.append( '' )
    api_def = '\n'.join(api_def_L1)
    return api_def


def generate_pkg_api_def( pkg_ns, pkg_base ):
    target_module_L1 = fs_mod.iter_py_mod( pkg_ns, pkg_base )
    L2 = [
            generate_api_def( target_module, pkg_base, target_only=True )
            for target_module, fspath in target_module_L1
        ]
    return '\n'.join(L2)


def read_relative_skeleton(target_module, module_dir):
    skeleton = parse_L0_skeleton.select_code_skeletons(target_module, module_dir)
    return skeleton


def p1():
    target_module = 'lib_dsa.graph_db'
    module_dir = '/home/algorithmspath/vol1/py_lib_src'
    pkg_ns = 'lib_compute'
    pkg_base = '/home/algorithmspath/vol1/py_lib_src'
    skeleton = read_relative_skeleton(target_module, module_dir)
    S1 = json.dumps(skeleton['lib_dsa.kv_iter'], indent=4)
    print(S1)


def main():
    p1()
    pass


if __name__ == '__main__':
    main()



import lib_compute.typon_math as typon_math

def read_bytes( fp, num_bytes, buffer_size ):
    if num_bytes >= buffer_size:
        return fp.read(num_bytes)
    return fp.read(buffer_size)


class forward_reader():
    def __init__( self, file_name, buffer_size ):
        self.file_name = file_name
        self.buffer_size = buffer_size
        self.buffer_index = 0
        self.entry_size = -1
        self.fp = open(file_name, 'rb')
        self.buffer = self.fp.read(buffer_size)
        self.iter_complete = False
    def has_next(self):
        if self.iter_complete:
            return False
        if self.entry_size > -1:
            return True
        if len( self.buffer ) < self.buffer_index + 8:
            bytes_to_read = self.buffer_index + 8 - len(self.buffer)
            buf_seg = read_bytes( self.fp, bytes_to_read, self.buffer_size )
            if len(buf_seg) < bytes_to_read:
                self.iter_complete = True
                return False
            if self.buffer_index > 0:
                self.buffer = self.buffer[ self.buffer_index : len(self.buffer) ]
                self.buffer_index = 0
            self.buffer += buf_seg
        size_repr = self.buffer[ self.buffer_index : self.buffer_index + 8 ]
        entry_size = typon_math.deserialize_int(size_repr)
        self.buffer_index += 8
        if len( self.buffer ) < self.buffer_index + entry_size:
            bytes_to_read = self.buffer_index + entry_size - len(self.buffer)
            buf_seg = read_bytes( self.fp, bytes_to_read, self.buffer_size )
            if len(buf_seg) < bytes_to_read:
                self.iter_complete = True
                return False
            if self.buffer_index > 0:
                self.buffer = self.buffer[ self.buffer_index : len(self.buffer) ]
                self.buffer_index = 0
            self.buffer += buf_seg
        entry_data = self.buffer[ self.buffer_index : self.buffer_index + entry_size ]
        self.entry_size = entry_size
        return True
    def get_next(self):
        if not self.has_next():
            return [ -1, -1 ]
        result = [ self.buffer_index, self.entry_size ]
        self.buffer_index += self.entry_size
        self.entry_size = -1
        return result
    def get_next_str(self):
        initial_index, entry_size = self.get_next()
        return self.buffer[ initial_index : initial_index + entry_size ]



import lib_compute.graph_alg as graph_alg

def fmt_mod_py( module_name, base_dir='' ):
    C1 = module_name.replace('.', '/')
    if len(base_dir) > 0:
        return f'{base_dir}/{C1}.py'
    return f'{C1}.py'


def iter_py_mod( pkg_ns, pkg_base ):
    L1 = graph_alg.bfs_files( f'{pkg_base}/{pkg_ns}', ['py'] )
    result = [
            [
                fspath[len(pkg_base)+1:].replace('/', '.')[:-3],
                fspath,
            ]
            for fspath in L1
        ]
    return result



import sys
import re
import lib_py_parse.utils.parsing_utils as parsing_utils

def log_step_transform(file_src):
    code_statements = []
    index = 0
    current_line = ''
    while index < len(file_src):
        if file_src[index] == '\n':
            code_statements.append(current_line)
            current_line = ''
            index += 1
            continue
        if file_src[index] in ['\'', '\"']:
            target_index = parsing_utils.get_quote_closure(file_src, index)
            current_line += file_src[ index : target_index ]
            index = target_index
            continue
        if file_src[index] in '[{(':
            target_index = parsing_utils.get_char_closure(file_src, index+1)
            current_line += file_src[ index : target_index ]
            index = target_index
            continue
        current_line += file_src[index]
        index += 1
        continue
    if len(current_line) > 0:
        code_statements.append(current_line)
    result_lines = []
    for line in code_statements:
        ls_str = line.lstrip()
        s_str = line.strip()
        nl_index = line.find('\n')
        if ls_str[0:6] == 'class ' or ls_str[0:4] == 'fxn ':
            result_lines.append(line)
            continue
        if nl_index != -1 or len(line) == 0 or len(ls_str) == 0 or ls_str[0] == '#':
            result_lines.append(line)
            continue
        if ls_str[0:8] == 'include ' or ls_str[0:7] == 'import ':
            result_lines.append(line)
            continue
        line_indent = line[ 0 : len(line) - len(ls_str) ]
        s_str_esc = parsing_utils.escape_quote(s_str)
        print_stmt = f'{line_indent}print(\'{s_str_esc}\')'
        result_lines.append(print_stmt)
        result_lines.append(line)
    return '\n'.join(result_lines)


def log_step_transform_write(file_name):
    fp = open(file_name, 'r')
    fsource = fp.read()
    fp.close()
    log_step_source = log_step_transform(fsource)
    print(log_step_source)
    fp = open(file_name, 'w')
    fp.write(log_step_source)
    fp.close()


def supress_log_step_write(file_name):
    fp = open(file_name, 'r')
    fsource = fp.read()
    fp.close()
    new_source = fsource.replace('print(', '# print(')
    print(new_source)
    fp = open(file_name, 'w')
    fp.write(new_source)
    fp.close()


def invert_log_step_transform( fsource ):
    regex_src = r'\n(\s+)print\((.*)\)'
    regex_replace = r''
    new_source = re.sub(regex_src, regex_replace, fsource)
    return new_source


def remove_print_statements(file_name):
    fp = open(file_name, 'r')
    fsource = fp.read()
    fp.close()
    new_source = invert_log_step_transform(fsource)
    fp = open(file_name, 'w')
    fp.write(new_source)
    fp.close()


def p1():
    file_name = '/home/algorithmspath/sdev/hb_api_typon/api/_api_generic/distributed_linked_list_api/insert_utils.typ'
    file_name = '/home/algorithmspath/sdev/hb_api_typon/api/select_hypothesis/selection/hypothesis_module.typ'
    file_name = '/home/algorithmspath/sdev/hb_api_typon/api/list_hypotheses/list_utils.typ'
    if len(sys.argv) < 3:
        print(f'usage: {sys.argv[0]} ' + ' {target_file_name} {log_step_transform|remove_print_statements}')
        return
    argument_names = [
            'target_file_name',
            'log_step_transform|remove_print_statements'
        ]
    status_str_L1 = [
            f'{argument_names[i]}: {sys.argv[i+1]}'
            for i in range(0, len(argument_names))
        ]
    status_str = 'log_step_transform|remove_print_statements:\n' + '\n'.join(status_str_L1)
    print(status_str)
    print()
    file_name = sys.argv[1]
    op_code = sys.argv[2]
    if op_code == '0':
        log_step_transform_write(file_name)
        return
    if op_code == '1':
        remove_print_statements(file_name)
        return
    print('no_op')


def main():
    p1()
    pass


if __name__ == '__main__':
    main()



import threading

class temp_worker(threading.Thread):
    def __init__(self, map_result_L1, thread_id, map_fxn, args):
        threading.Thread.__init__(self)
        self.map_result_L1 = map_result_L1
        self.thread_id = thread_id
        self.map_fxn = map_fxn
        self.args = args
    def run(self):
        self.map_result_L1[self.thread_id] = self.map_fxn(*self.args)


def map_reduce( reduce_fxn, map_fxn_L1, map_args_L1 ):
    map_result_L1 = {}
    workers = [
            temp_worker( map_result_L1, i, map_fxn_L1[i], map_args_L1[i] )
            for i in range(0, len(map_fxn_L1))
        ]
    for i in range(0, len(map_fxn_L1)):
        workers[i].start()
    for i in range(0, len(map_fxn_L1)):
        workers[i].join()
    reduce_fxn(map_result_L1)



import time
import requests
from bs4 import BeautifulSoup
import lib_dsa.byte_array as byte_array
import lib_compute.serialization as serialization
import lib_compute.deserialization as deserialization
import selenium
from selenium import webdriver
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.firefox.options import Options
import pandas as pd
import random

def read_website(url):
    resp = requests.get(url)
    return resp.content


def scrape_L1(links):
    return [
            read_website(url)
            for url in links
        ]


def read_iter(file_names, mode='rb'):
    result = []
    for file_name in file_names:
        fp = open(file_name, mode)
        fsource = fp.read()
        fp.close()
        result.append(fsource)
    return result


def scrape_write(link, file_name):
    file_repr = read_website(link)
    fp = open(file_name, 'wb')
    fp.write(file_repr)
    fp.close()


def scrape_write_L1(links, out_files):
    for i in range(0, len(links)):
        print(i)
        scrape_write(links[i], out_files[i])


def fwrite_L1(file_name_L1, file_repr_L1, mode='wb'):
    i = 0
    for file_name, file_repr in zip(file_name_L1, file_repr_L1):
        print( i )
        i += 1
        fp = open(file_name, mode)
        fp.write(file_repr)
        fp.close()


def select_links(html_raw):
    soup = BeautifulSoup(html_raw, 'html.parser')
    L1 = soup.find_all('a')
    result = []
    for x in L1:
        if 'href' not in x.attrs:
            continue
        result.append( x.attrs['href'] )
    return result


def vba_write(base_path, str_L1, is_bytes=False):
    vba = byte_array.make_vba(base_path)
    if is_bytes:
        vba.batch_insert_bytes(str_L1)
        return
    target_bytes_L1 = []
    try:
        target_bytes_L1 = [ x.encode('latin-1') for x in str_L1 ]
    except:
        target_bytes_L1 = [ x.encode() for x in str_L1 ]
    vba.batch_insert_bytes(target_bytes_L1)


def eval_html_srcs(file_name_L1, out_file_L1, time_sleep=1, executable_path=''):
    options = Options()
    options.headless = True
    driver = webdriver.Firefox(options=options, executable_path=executable_path)
    for fspath, out_file in zip(file_name_L1, out_file_L1):
        file_name = f'file://{fspath}'
        print(file_name)
        try:
            X = driver.get(file_name)
            time.sleep(time_sleep)
            Y = driver.page_source
        except:
            continue
        fp = open(out_file, 'w')
        fp.write(Y)
        fp.close()
    driver.quit()



import os
import sys
import lib_code.parse_imports as parse_imports

def read_src(file_path):
    fp = open(file_name, 'r')
    src_code = fp.read()
    fp.close()
    return src_code


def rm_pkg_ns_typ(file_path, pkg_name, target_modules):
    src_code = read_src(file_path)
    include_set, aliased_imports, code_minus_imports = parse_imports.split_imports_typ(src_code)
    for module_name in target_modules:
        if module_name not in aliased_imports:
            continue
        if module_name[0:len(pkg_name)+1] != pkg_name + '.':
            continue
        module_alias = aliased_imports[module_name]
        del aliased_imports[module_name]
        trunc_mod_name = module_name[len(pkg_name)+1:]
        aliased_imports[trunc_mod_name] = module_alias
    return parse_imports.join_imports_typ(include_set, aliased_imports, code_minus_imports)


def rm_pkg_ns_py(file_path, pkg_name, target_modules):
    src_code = read_src(file_path)
    sourced_imports, aliased_imports, code_minus_imports = parse_imports.split_imports_py(src_code)
    for module_name in target_modules:
        if module_name not in aliased_imports:
            continue
        if module_name[0:len(pkg_name)+1] != pkg_name + '.':
            continue
        module_alias = aliased_imports[module_name]
        del aliased_imports[module_name]
        trunc_mod_name = module_name[len(pkg_name)+1:]
        aliased_imports[trunc_mod_name] = module_alias
    for module_name in target_modules:
        if module_name not in sourced_imports:
            continue
        if module_name[0:len(pkg_name)+1] != pkg_name + '.':
            continue
        module_data = sourced_imports[module_name]
        del sourced_imports[module_name]
        trunc_mod_name = module_name[len(pkg_name)+1:]
        sourced_imports[trunc_mod_name] = module_data
    return parse_imports.join_imports_py(sourced_imports, aliased_imports, code_minus_imports)


def append_pkg_ns_typ(file_path, pkg_name, target_modules):
    src_code = read_src(file_path)
    include_set, aliased_imports, code_minus_imports = parse_imports.split_imports_typ(src_code)
    for module_name in target_modules:
        if module_name not in aliased_imports:
            continue
        module_alias = aliased_imports[module_name]
        del aliased_imports[module_name]
        aliased_imports[f'{pkg_name}.{module_name}'] = module_alias
    return parse_imports.join_imports_typ(include_set, aliased_imports, code_minus_imports)


def append_pkg_ns_py(file_path, pkg_name, target_modules):
    src_code = read_src(file_path)
    sourced_imports, aliased_imports, code_minus_imports = parse_imports.split_imports_py(src_code)
    for module_name in target_modules:
        if module_name not in aliased_imports:
            continue
        module_alias = aliased_imports[module_name]
        del aliased_imports[module_name]
        aliased_imports[f'{pkg_name}.{module_name}'] = module_alias
    for module_name in target_modules:
        if module_name not in sourced_imports:
            continue
        module_data = sourced_imports[module_name]
        del sourced_imports[module_name]
        sourced_imports[f'{pkg_name}.{module_name}'] = module_data
    return parse_imports.join_imports_py(sourced_imports, aliased_imports, code_minus_imports)


def change_imports_typ(file_path, imports_to_add, imports_to_remove):
    src_code = read_src(file_path)
    include_set, aliased_imports, code_minus_imports = parse_imports.split_imports_typ(src_code)
    for module_name, module_alias in imports_to_add:
        aliased_imports[module_name] = module_alias
    for module_name in imports_to_remove:
        if module_name in imports_set:
            del aliased_imports[module_name]
    return parse_imports.join_imports_typ(include_set, aliased_imports, code_minus_imports)


def change_imports_py(file_path, imports_to_add, imports_to_remove):
    src_code = read_src(file_path)
    sourced_imports, aliased_imports, code_minus_imports = parse_imports.split_imports_py(src_code)
    for module_name, module_alias in imports_to_add:
        aliased_imports[module_name] = module_alias
    for module_name in imports_to_remove:
        if module_name in imports_set:
            del aliased_imports[module_name]
    return parse_imports.join_imports_py(sourced_imports, aliased_imports, code_minus_imports)



import setuptools

pkg_name = 'libSDT8'
version = '0.0.0'

setuptools.setup(
    name=pkg_name,
    version=version,
    description='Local packages for development',

    author='hypothesisbase',
    author_email="support@hypothesisbase.com",
    url= f'http://pypi.python.org/pypi/{pkg_name}/{version}',
    license="MIT",

    install_requires=[
        "bs4",
        "leveldb",
        "selenium",
        "requests",
        "pandas"
    ],
    packages=setuptools.find_packages(),
    zip_safe=False
)



import os
import traceback
import lib_py_parse.parse.parser as parser
import lib_py_parse.utils.fmt_utils as fmt_utils
import lib_py_parse.utils.exceptions as exceptions
import lib_py_parse.helper.eval_helper as eval_helper
import lib_py_parse.helper.eval_service as eval_service

def select_modules_to_read(item_skeleton):
    L1 = [
            x for x in item_skeleton
            if x['type'] == 'import'
        ]
    result = []
    for T1 in L1:
        T2 = [
                    T3.strip()
                    for T3 in T1['expression'][6:].split(' as ')
                ]
        if len(T2) != 2:
            continue
        module_name, module_alias = T2
        result.append(module_name)
    return result


def read_import_to_skeleton(item, module_L0_skeleton, current_module):
    T1 = [
            T2.strip()
            for T2 in item['expression'][6:].split(' as ')
        ]
    assert len(T1) == 2
    module_name, module_alias = T1
    module_L0_skeleton['imports']['alias_map'][module_alias] = module_name


def read_L0_skeleton(current_module, item_skeleton):
    try:
        modules_to_read = select_modules_to_read(item_skeleton)
        module_L0_skeleton = item_skeleton
    except:
        e_msg = f'''
        error reading module: {current_module}
        error_msg: {str(e)}
        '''
        traceback.print_exc()
        exit(1)
    return modules_to_read, module_L0_skeleton


def fmt_code(src_dir):
    S1 = open(src_dir, 'r').read()
    L1 = S1.splitlines()
    L2 = []
    for line in L1:
        l1 = line.lstrip()
        if len(l1) == 0:
            L2.append(l1)
            continue
        if l1[0] == '#':
            L2.append('')
            continue
        L2.append(line)
    L2.append('')
    result = '\n'.join(L2)
    return result


def select_code_skeletons(target_module, module_dir):
    q = [target_module]
    L0_skeleton = {}
    visited_modules = {}
    included_c_exts_map = {}
    while len(q) > 0:
        current_module = q.pop()
        if current_module in visited_modules:
            continue
        try:
            src_file = fmt_utils.select_module_src_dir(module_dir, current_module)
        except:
            continue
        fp = open(src_file, 'r')
        code = fp.read()
        fp.close()
        try:
            item_skeleton, end_index = parser.read_scope(code, 0, 0)
        except:
            traceback.print_exc()
            print(src_file)
            exit(1)
        modules_to_read, module_L0_skeleton = read_L0_skeleton(current_module, item_skeleton)
        visited_modules[current_module] = 1
        q += [
                    tgt_module
                    for tgt_module in modules_to_read
                    if tgt_module not in visited_modules
                ]
        L0_skeleton[current_module] = module_L0_skeleton
    return L0_skeleton



import os
import lib_dsa.kv_db as kv_db
import lib_compute.typon_math as typon_math

class graph_edge_iterator():
    def __init__(self, gs_ref, src_node_bytes):
        self.gs_ref = gs_ref
        self.src_node_bytes = src_node_bytes
        self.edge_it = gs_ref.iterator(
                    prefix=src_node_bytes,
                    include_key=True,
                    include_value=True,
                    reverse=False
                )
        self.tgt_node_bytes = b''
        self.val_set = False
    def has_next(self):
        if self.val_set:
            return True
        if not self.edge_it.has_next():
            return False
        while self.edge_it.has_next():
            potential_edge, evalue = self.edge_it.get_next()
            src_len = typon_math.deserialize_int32( potential_edge[ len(potential_edge)-4 : ] )
            pt_src = potential_edge[ 0 : src_len ]
            if pt_src == self.src_node_bytes:
                self.tgt_node_bytes = potential_edge[ src_len : len(potential_edge)-4 ]
                self.val_set = True
                return True
        return False
    def get_next(self):
        if not self.has_next():
            return b''
        self.val_set = False
        return self.tgt_node_bytes


class graph_store():
    def __init__(self, db_base, vba_engine=False):
        if not os.path.isdir(db_base):
            os.mkdir(db_base)
        node_table = f'{db_base}/n'
        edge_table = f'{db_base}/e'
        self.node_store = kv_db.kv_store( node_table, vba_engine=vba_engine )
        self.edge_store = kv_db.kv_store( edge_table, vba_engine=False )
    def node_batch_write(self, op_args_L1, transaction=False):
        return self.node_store.batch_write(op_args_L1, transaction=transaction)
    def set_node(self, node_key, node_value, key_encoding='latin-1', value_encoding='latin-1'):
        return self.node_store.set(node_key, node_value, key_encoding=key_encoding, value_encoding=value_encoding)
    def get_node(self, node_key, key_encoding='latin-1'):
        return self.node_store.get(node_key, key_encoding=key_encoding)
    def remove_node(self, node_key, key_encoding='latin-1'):
        return self.node_store.remove(node_key, key_encoding=key_encoding)
    def set_edge(self, src_node, tgt_node, src_encoding='latin-1', tgt_encoding='latin-1'):
        src_enc = src_node
        if isinstance(src_enc, str):
            src_enc = src_node.encode(src_encoding)
        tgt_enc = tgt_node
        if isinstance(tgt_enc, str):
            tgt_enc = tgt_node.encode(tgt_encoding)
        edge_repr = b'\x01'
        src_len_enc = typon_math.serialize_int32( len(src_node) )
        edge_key = src_enc + tgt_enc + src_len_enc
        return self.node_store.set(edge_key, edge_repr)
    def remove_edge(self, src_node, tgt_node, src_encoding='latin-1', tgt_encoding='latin-1'):
        src_enc = src_node
        if isinstance(src_enc, str):
            src_enc = src_node.encode(src_encoding)
        tgt_enc = tgt_node
        if isinstance(tgt_enc, str):
            tgt_enc = tgt_node.encode(tgt_encoding)
        src_len_enc = typon_math.serialize_int32( len(src_node) )
        edge_key = src_enc + tgt_enc + src_len_enc
        return self.node_store.remove(edge_key)
    def edge_iterator(self, src_node, src_encoding='latin-1'):
        src_node_bytes = src_node
        if isinstance(src_node, str):
            src_node_bytes = src_node.encode(src_encoding)
        return graph_edge_iterator( self, src_node_bytes )
    def iterate_edges(self, src_node, src_encoding='latin-1'):
        src_node_bytes = src_node
        if isinstance(src_node, str):
            src_node_bytes = src_node.encode(src_encoding)
        result = []
        edge_it = graph_edge_iterator( self, src_node_bytes )
        while edge_it.has_next():
            result.append( edge_it.get_next() )
        return result
